/*
 * Operation for array of tree type
 * Teodor Sigaev <teodor@stack.net>
 */


#include "postgres.h"

#include "access/gist.h"
#include "access/itup.h"
#include "access/rtree.h"
#include "utils/elog.h"
#include "utils/palloc.h"
#include "utils/array.h"
#include "utils/builtins.h"
#include "storage/bufpage.h"
#include "access/nbtree.h"

#include "entree.h"

PG_FUNCTION_INFO_V1(_tq_eq);
Datum   _tq_eq(PG_FUNCTION_ARGS);
PG_FUNCTION_INFO_V1(_entq_req);
Datum   _entq_req(PG_FUNCTION_ARGS);
PG_FUNCTION_INFO_V1(_entq_contains);
Datum   _entq_contains(PG_FUNCTION_ARGS);
PG_FUNCTION_INFO_V1(_entq_contain);
Datum   _entq_contain(PG_FUNCTION_ARGS);

static bool
inner_arrtq_eq( ArrayType *node, entreenode *q ) {
	entreenode *cur = ARRPTR(node);
	int4 len = ARRNELEMS(node);
	register int4 i=0;

	if ( len<=0 )
		return false;

	if ( cmp_ennode(cur,q) == 0 )
		return true;

	for(i=1;i<len;i++) {
		cur = NEXTVAL(cur);
		if ( cmp_ennode(cur,q) == 0 )
			return true;
	}

	return false;		
}

Datum   
_entq_req(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(1) );
	bool res;

	res = inner_arrtq_eq(
		arrnode,
		(entreenode*) PG_GETARG_POINTER(0)
	);
	PG_FREE_IF_COPY(arrnode, 1);
	PG_RETURN_BOOL(res);
}

Datum   
_tq_eq(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(0) );
	bool res;
	
	res = inner_arrtq_eq(
		arrnode,
		(entreenode*) PG_GETARG_POINTER(1)
	);
	PG_FREE_IF_COPY(arrnode, 0);
	PG_RETURN_BOOL(res);
}

static bool
inner_arrentq_contain( ArrayType *node, treequery *mask ) {
	entreenode *cur = ARRPTR(node);
	int4 len = ARRNELEMS(node);
	register int4 i=0;

	if ( len<=0 )
		return false;

	if ( inner_entq_contain(cur,mask) )
		return true;
	for(i=1;i<len;i++) {
		cur = NEXTVAL(cur);
		if ( inner_entq_contain(cur,mask) )
			return true;
	}
	return false;		
}

Datum   
_entq_contains(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(1) );
	bool res;
	
	res = inner_arrentq_contain(
		arrnode,
		(treequery*) PG_GETARG_POINTER(0)
	);
	PG_FREE_IF_COPY(arrnode, 1);
	PG_RETURN_BOOL(res);
}

Datum   
_entq_contain(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(0) );
	bool res;
	
	res = inner_arrentq_contain(
		arrnode,
		(treequery*) PG_GETARG_POINTER(1)
	);
	PG_FREE_IF_COPY(arrnode, 0);
	PG_RETURN_BOOL(res);
}


