\set ECHO none
\i tree.sql
\set ECHO all

--test bitree type

select ''::bitree;
select '1'::bitree;
select '1.2'::bitree;
select '1.2.3'::bitree;

--test bitreequery type

select '1'::bitreequery;
select '4|3|2'::bitreequery;
select '1.2'::bitreequery;
select '1.4|3|2'::bitreequery;
select '1.0'::bitreequery;
select '4|3|2.0'::bitreequery;
select '1.2.0'::bitreequery;
select '1.4|3|2.0'::bitreequery;
select '1.*'::bitreequery;
select '4|3|2.*'::bitreequery;
select '1.2.*'::bitreequery;
select '1.4|3|2.*'::bitreequery;
select '*.1.*'::bitreequery;
select '*.4|3|2.*'::bitreequery;
select '*.1.2.*'::bitreequery;
select '*.1.4|3|2.*'::bitreequery;
select '1.*.4|3|2'::bitreequery;
select '1.*.4|3|2.0'::bitreequery;
select '1.*.4|3|2.*'::bitreequery;

--test operations
select bitree_level('1.2.3.4');
select bitree_next('1.2.3.4');
select '1.2'::bitree  < '2.2'::bitree;
select '1.2'::bitree  <= '2.2'::bitree;
select '2.2'::bitree  = '2.2'::bitree;
select '3.2'::bitree  >= '2.2'::bitree;
select '3.2'::bitree  > '2.2'::bitree;

select '1.2.3.4.5'::bitree <* '1.1|2.*.6|5|4.5.0'::bitreequery;
select '1.2.3.4.5'::bitree <* '1.1|2.*.6|5.5.0'::bitreequery;

select '1.2.3'::bitree @> '1.2.3.4';
select '1.2.3.4'::bitree @> '1.2.3.4';
select '1.2.3.4.5'::bitree @> '1.2.3.4';
select '1.3.3'::bitree @> '1.2.3.4';

create table bitreetest (t bitree);
\copy bitreetest from 'data/tree.data'

select * from bitreetest where t <  '12.3' order by t asc;
select * from bitreetest where t <= '12.3' order by t asc;
select * from bitreetest where t =  '12.3' order by t asc;
select * from bitreetest where t >= '12.3' order by t asc;
select * from bitreetest where t >  '12.3' order by t asc;
select * from bitreetest where t <* '12|23|14|16.*.22|11|23' order by t asc;
select * from bitreetest where t @> '1.1.1' order by t asc;
select * from bitreetest where t <@ '1.1.1' order by t asc;

create unique index tstidx on bitreetest (t);
set enable_seqscan=off;

select * from bitreetest where t <  '12.3' order by t asc;
select * from bitreetest where t <= '12.3' order by t asc;
select * from bitreetest where t =  '12.3' order by t asc;
select * from bitreetest where t >= '12.3' order by t asc;
select * from bitreetest where t >  '12.3' order by t asc;
select * from bitreetest where t <* '12|23|14|16.*.22|11|23' order by t asc;
select * from bitreetest where t @> '1.1.1' order by t asc;
select * from bitreetest where t <@ '1.1.1' order by t asc;

drop index tstidx;
create index tstidx on bitreetest using gist (t);
set enable_seqscan=off;

select * from bitreetest where t <  '12.3' order by t asc;
select * from bitreetest where t <= '12.3' order by t asc;
select * from bitreetest where t =  '12.3' order by t asc;
select * from bitreetest where t >= '12.3' order by t asc;
select * from bitreetest where t >  '12.3' order by t asc;
select * from bitreetest where t <* '12|23|14|16.*.22|11|23' order by t asc;
select * from bitreetest where t @> '1.1.1' order by t asc;
select * from bitreetest where t <@ '1.1.1' order by t asc;

--test arrays
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_bitree => '3.4.5.6'::bitree;
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_bitree => '3.4.5.7'::bitree;
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_bitree <* '3.4.5'::bitreequery;
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_bitree <* '3.4.5.0'::bitreequery;

create table bitreearr ( t bitree[] );
\copy bitreearr from 'data/_tree.data'

select count(*) from bitreearr where t => '7.20';
select count(*) from bitreearr where t <* '7.20';

create index bitreearridx on bitreearr using gist ( t );

select count(*) from bitreearr where t => '7.20';
select count(*) from bitreearr where t <* '7.20';

--test entree type

select ''::entree;
select '1'::entree;
select '1.2'::entree;
select '1.2.3'::entree;

--test entreequery type

select '1'::entreequery;
select '4|3|2'::entreequery;
select '1.2'::entreequery;
select '1.4|3|2'::entreequery;
select '1.0'::entreequery;
select '4|3|2.0'::entreequery;
select '1.2.0'::entreequery;
select '1.4|3|2.0'::entreequery;
select '1.*'::entreequery;
select '4|3|2.*'::entreequery;
select '1.2.*'::entreequery;
select '1.4|3|2.*'::entreequery;
select '*.1.*'::entreequery;
select '*.4|3|2.*'::entreequery;
select '*.1.2.*'::entreequery;
select '*.1.4|3|2.*'::entreequery;
select '1.*.4|3|2'::entreequery;
select '1.*.4|3|2.0'::entreequery;
select '1.*.4|3|2.*'::entreequery;

--test operations
select entree_level('1.2.3.4');
select entree_next('1.2.3.4');
select '1.2'::entree  < '2.2'::entree;
select '1.2'::entree  <= '2.2'::entree;
select '2.2'::entree  = '2.2'::entree;
select '3.2'::entree  >= '2.2'::entree;
select '3.2'::entree  > '2.2'::entree;

select '1.2.3.4.5'::entree <* '1.1|2.*.6|5|4.5.0'::entreequery;
select '1.2.3.4.5'::entree <* '1.1|2.*.6|5.5.0'::entreequery;

select '1.2.3'::entree @> '1.2.3.4';
select '1.2.3.4'::entree @> '1.2.3.4';
select '1.2.3.4.5'::entree @> '1.2.3.4';
select '1.3.3'::entree @> '1.2.3.4';

create table entreetest (t entree);
\copy entreetest from 'data/tree.data'

select * from entreetest where t <  '12.3' order by t asc;
select * from entreetest where t <= '12.3' order by t asc;
select * from entreetest where t =  '12.3' order by t asc;
select * from entreetest where t >= '12.3' order by t asc;
select * from entreetest where t >  '12.3' order by t asc;
select * from entreetest where t <* '12|23|14|16.*.22|11|23' order by t asc;
select * from entreetest where t @> '1.1.1' order by t asc;
select * from entreetest where t <@ '1.1.1' order by t asc;

create unique index entstidx on entreetest (t);
set enable_seqscan=off;

select * from entreetest where t <  '12.3' order by t asc;
select * from entreetest where t <= '12.3' order by t asc;
select * from entreetest where t =  '12.3' order by t asc;
select * from entreetest where t >= '12.3' order by t asc;
select * from entreetest where t >  '12.3' order by t asc;
select * from entreetest where t <* '12|23|14|16.*.22|11|23' order by t asc;
select * from entreetest where t @> '1.1.1' order by t asc;
select * from entreetest where t <@ '1.1.1' order by t asc;

drop index entstidx;
create index entstidx on entreetest using gist (t);
set enable_seqscan=off;

select * from entreetest where t <  '12.3' order by t asc;
select * from entreetest where t <= '12.3' order by t asc;
select * from entreetest where t =  '12.3' order by t asc;
select * from entreetest where t >= '12.3' order by t asc;
select * from entreetest where t >  '12.3' order by t asc;
select * from entreetest where t <* '12|23|14|16.*.22|11|23' order by t asc;
select * from entreetest where t @> '1.1.1' order by t asc;
select * from entreetest where t <@ '1.1.1' order by t asc;

--test arrays
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_entree => '3.4.5.6'::entree;
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_entree => '3.4.5.7'::entree;
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_entree <* '3.4.5'::entreequery;
select '{1.2.3.4,2.3.4.5,3.4.5.6}'::_entree <* '3.4.5.0'::entreequery;

create table entreearr ( t entree[] );
\copy entreearr from 'data/_tree.data'

select count(*) from entreearr where t => '7.20';
select count(*) from entreearr where t <* '7.20';

create index entreearridx on entreearr using gist ( t );

select count(*) from entreearr where t => '7.20';
select count(*) from entreearr where t <* '7.20';

